"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformDtsExtensions = exports.transform = void 0;
var _promises = require("node:fs/promises");
var _parser = require("@babel/parser");
var _core = require("@babel/core");
var _traverse = _interopRequireDefault(require("@babel/traverse"));
var _magicString = _interopRequireDefault(require("magic-string"));
var _util = require("./util.cjs");
var _helpers = require("./helpers.cjs");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const traverse = _traverse.default.default;
const transform = async (filename, opts) => {
  const source = await (0, _promises.readFile)(filename);
  const esmSource = source.toString();
  const cjsSource = new _magicString.default(esmSource);
  const {
    esmPresets,
    esmPlugins,
    cjsPresets,
    ...rest
  } = opts;
  const baseOpts = {
    ...rest,
    babelrc: false,
    configFile: false
  };
  const {
    ast,
    code: esm,
    map: esmMap
  } = await (0, _core.transformAsync)(esmSource, {
    ...baseOpts,
    plugins: esmPlugins,
    presets: esmPresets
  });
  traverse(ast, {
    ImportDeclaration(path) {
      (0, _helpers.updateExtensionsToCjs)(cjsSource, path);
    },
    CallExpression(path) {
      if (path.get('callee').isImport()) {
        (0, _helpers.updateExtensionsToCjs)(cjsSource, path, 'arguments.0');
      }
    },
    'ExportNamedDeclaration|ExportAllDeclaration'(path) {
      (0, _helpers.updateExtensionsToCjs)(cjsSource, path);
    }
  });
  const {
    code: cjs,
    map: cjsMap
  } = await (0, _core.transformAsync)(cjsSource.toString(), {
    ...baseOpts,
    ast: false,
    presets: (0, _util.isEsModuleFile)(filename) ? esmPresets : cjsPresets
  });
  return {
    code: {
      esm,
      cjs
    },
    map: {
      esm: esmMap,
      cjs: cjsMap
    }
  };
};
exports.transform = transform;
const transformDtsExtensions = async filename => {
  const source = (await (0, _promises.readFile)(filename)).toString();
  const dtsCjsSrc = new _magicString.default(source);
  const ast = (0, _parser.parse)(source, {
    sourceType: 'module',
    plugins: [['typescript', {
      dts: true
    }]]
  });
  traverse(ast, {
    TSImportType(path) {
      (0, _helpers.updateExtensionsToCjs)(dtsCjsSrc, path, 'argument');
    },
    ImportDeclaration(path) {
      (0, _helpers.updateExtensionsToCjs)(dtsCjsSrc, path);
    },
    'ExportNamedDeclaration|ExportAllDeclaration'(path) {
      (0, _helpers.updateExtensionsToCjs)(dtsCjsSrc, path);
    }
  });
  return dtsCjsSrc.toString();
};
exports.transformDtsExtensions = transformDtsExtensions;