"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.replaceJsExtWithCjs = exports.logResult = exports.logNotice = exports.logHelp = exports.logError = exports.log = exports.isRelative = exports.isEsModuleFile = exports.hasJsExt = exports.getRealPathAsFileUrl = exports.getPresetIdx = exports.getPluginIdx = exports.getOutFileExt = exports.getModulePresets = exports.getListWithItemRemoved = exports.getFiles = exports.getEsmPlugins = exports.getConfigItem = exports.addDefaultPresets = void 0;
var _nodeUrl = require("node:url");
var _nodePath = require("node:path");
var _promises = require("node:fs/promises");
var _core = require("@babel/core");
const _filename = (0, _nodeUrl.fileURLToPath)(import.meta.url);
const _dirname = (0, _nodePath.dirname)(_filename);
const red = '\x1b[31m';
const cyan = '\x1b[36m';
const black = '\x1b[30m';
const jsExtRegex = /\.(js['"`\s]*?)$/i;
const relativeSpecifierRegex = /^['"`\s]*?(?:\.|\.\.)\//i;
const log = (color = cyan, msg = '', prefix = '') => {
  // eslint-disable-next-line no-console
  console.log(`${color}%s\x1b[0m`, `${prefix}${msg}`);
};
exports.log = log;
const logError = log.bind(null, red);
exports.logError = logError;
const logNotice = log.bind(null, cyan);
exports.logNotice = logNotice;
const logResult = log.bind(null, black);
exports.logResult = logResult;
const logHelp = msg => {
  logResult(msg, '');
};
exports.logHelp = logHelp;
const getRealPathAsFileUrl = async path => {
  const realPath = await (0, _promises.realpath)(path);
  const asFileUrl = (0, _nodeUrl.pathToFileURL)(realPath).href;
  return asFileUrl;
};
exports.getRealPathAsFileUrl = getRealPathAsFileUrl;
const getFiles = async dir => {
  const dirents = await (0, _promises.readdir)(dir, {
    withFileTypes: true
  });
  const files = await Promise.all(dirents.map(dirent => {
    const path = (0, _nodePath.resolve)(dir, dirent.name);
    return dirent.isFile() ? path : getFiles(path);
  }));
  return files.flat();
};
exports.getFiles = getFiles;
const getConfigItem = (value, type = 'preset') => {
  return (0, _core.createConfigItem)(value, {
    type,
    dirname: _dirname
  });
};
exports.getConfigItem = getConfigItem;
const getPresetIdx = (presets, preset) => {
  return presets.findIndex(pre => {
    return new RegExp(`@babel/preset-${preset}`).test(pre.file?.resolved);
  });
};
exports.getPresetIdx = getPresetIdx;
const getPluginIdx = (plugins, plugin) => {
  return plugins.findIndex(plug => {
    return new RegExp(`@babel/plugin-${plugin}`).test(plug.file?.resolved);
  });
};
exports.getPluginIdx = getPluginIdx;
const getListWithItemRemoved = (list, item, type) => {
  const gettersIdx = {
    preset: () => getPresetIdx(list, item),
    plugin: () => getPluginIdx(list, item)
  };
  const idx = gettersIdx[type]();
  if (idx > -1) {
    const clone = [...list];
    clone.splice(idx, 1);
    return clone;
  }
  return list;
};
exports.getListWithItemRemoved = getListWithItemRemoved;
const addDefaultPresets = (presets, extensions) => {
  const shouldDisallow = /\.(?:mts|cts|tsx)/i.test(extensions.join(''));
  const isTsx = extensions.includes('.tsx');
  const allExtensions = isTsx || shouldDisallow ? true : false;
  logNotice('No presets found, using default presets: ["@babel/preset-env", "@babel/preset-typescript", "@babel/preset-react"]');
  presets.push(getConfigItem('@babel/preset-env'), getConfigItem(['@babel/preset-typescript', {
    allExtensions,
    allowDeclareFields: true,
    isTSX: isTsx,
    disallowAmbiguousJSXLike: shouldDisallow
  }]), getConfigItem(['@babel/preset-react', {
    runtime: 'automatic'
  }]));
};
exports.addDefaultPresets = addDefaultPresets;
const getModulePresets = (presets, type = 'esm') => {
  const clones = [...presets];
  const modules = type === 'esm' ? false : 'commonjs';
  const envPresetIdx = clones.findIndex(preset => /@babel\/preset-env/.test(preset.file.resolved));
  if (envPresetIdx > -1) {
    clones.splice(envPresetIdx, 1, getConfigItem(['@babel/preset-env', {
      ...clones[envPresetIdx].options,
      modules
    }]));
  } else {
    clones.unshift(getConfigItem(['@babel/preset-env', {
      modules
    }]));
  }
  return clones;
};
exports.getModulePresets = getModulePresets;
const getEsmPlugins = plugins => {
  const clones = [...plugins];
  const cjsPluginIdx = clones.findIndex(plugin => /@babel\/plugin-transform-modules-commonjs/.test(plugin.file.resolved));
  if (cjsPluginIdx > -1) {
    // Remove this plugin and use preset-env `modules` option instead
    clones.splice(cjsPluginIdx, 1);
  }
  return clones;
};
exports.getEsmPlugins = getEsmPlugins;
const hasJsExt = str => jsExtRegex.test(str);
exports.hasJsExt = hasJsExt;
const replaceJsExtWithCjs = str => {
  const idx = str.lastIndexOf('.js');
  const step = '.js'.length;
  if (idx > -1) {
    const newExt = '.cjs';
    return `${str.substring(0, idx)}${newExt}${str.substring(idx + step, str.length)}`;
  }
  return str;
};
exports.replaceJsExtWithCjs = replaceJsExtWithCjs;
const isRelative = str => {
  // Obvious relative, or starts with a template string and has .js ext.
  return relativeSpecifierRegex.test(str) || /^`\${/i.test(str) && hasJsExt(str);
};
exports.isRelative = isRelative;
const isEsModuleFile = str => /\.m[jt]s?/.test(str);
exports.isEsModuleFile = isEsModuleFile;
const getOutFileExt = (ext, outFileExtension, keepFileExtension, type = 'esm') => {
  if (keepFileExtension) {
    return ext;
  }
  if (outFileExtension) {
    return outFileExtension[type];
  }
  if (/\.m[jt]s/.test(ext)) {
    return '.mjs';
  }
  if (/\.c[jt]s/.test(ext) || type === 'cjs') {
    return '.cjs';
  }
  return '.js';
};
exports.getOutFileExt = getOutFileExt;